import AsyncStorage from '@react-native-async-storage/async-storage';
import DeviceInfo from 'react-native-device-info';
import {API_URL, Traits} from '../utils/constants';
import {flush} from './flush';

import {NavigationContainerRef, ParamListBase} from '@react-navigation/native';
import {screen} from './screen';
// import {startNetworkListener} from './networkListner';
import {trackLifeCycleEvents} from './trackLifeCycleEvents';
import {startNetworkListener} from './networkListner';
import {getFingerprintId} from '../native/FingerprintNative';

let navigationRef: NavigationContainerRef<ParamListBase> | null = null;

export const setNavigationRef = (
  ref: NavigationContainerRef<ParamListBase>,
) => {
  navigationRef = ref;
};

const trackCurrentScreen = () => {
  if (navigationRef && instance?.trackScreenViews) {
    const currentRoute = navigationRef.getCurrentRoute();
    if (currentRoute) {
      console.log('currentRoute -> ', currentRoute.name);
      const {name} = currentRoute;
      screen(null, name);
    }
  }
};

const startScreenTracking = () => {
  if (navigationRef) {
    navigationRef.addListener('state', trackCurrentScreen);
  }
};

export let instance: {
  anonymousId: string | any;
  userId: string | null;
  apiKey: string | null;
  traits: Traits;
  groupId: string | null;
  eventQueue: any[];
  trackScreenViews: boolean;
  optOutStatus: boolean;

  identityId: any;
  sessionId: any;
  fingerprintId: any;

  flushInterval: number;
  flushQueueSize: number;
  apiHost: string;
  schemaVersion?: string;
} | null = null;

const DEFAULT_FLUSH_INTERVAL = 30 * 1000;
const DEFAULT_FLUSH_QUEUE_SIZE = 20;
const DEFAULT_API_URL = API_URL;

const setApiHost = (host: string) => {
  if (!instance) {
    console.error('SDK instance is not initialized.');
    return;
  }

  try {
    const urlParts = DEFAULT_API_URL.split('/');
    urlParts[2] = host;
    const updatedUrl = urlParts.join('/');
    // console.log('Final Updated Url', updatedUrl);
    instance!.apiHost = updatedUrl;
  } catch (error) {
    console.error('Error Setting API host: ', error);
  }
};

export const initializeSDK = async (
  apiKey: string,
  options: {
    trackScreenViews?: boolean;
    flushInterval?: number;
    flushQueueSize?: number;
    apiHost?: string;
    schemaVersion?: string;
  } = {},
) => {
  if (!instance) {
    const storedIdentityId = await getIdentityId();
    const fingerprintId = await getFingerprintId();
    console.log('FINGER PRINT ID----> ', fingerprintId)

    instance = {
      anonymousId: generateAnonymousId(),
      userId: null, // Will be null if not set, device Id
      apiKey: apiKey,
      traits: {},
      groupId: null,
      eventQueue: [],
      trackScreenViews: options?.trackScreenViews || false,
      optOutStatus: await getOptOutStatus(),
      flushInterval: options.flushInterval || DEFAULT_FLUSH_INTERVAL,
      flushQueueSize: options.flushQueueSize || DEFAULT_FLUSH_QUEUE_SIZE,
      identityId: storedIdentityId,
      sessionId: generateAnonymousId(), // Generate a new sessionId for this session
      fingerprintId: fingerprintId,
      apiHost: DEFAULT_API_URL,
      schemaVersion: options.schemaVersion || 'Default',
    };

    if (options.apiHost) {
      setApiHost(options.apiHost);
    }
  }

  const storedEventQueue = await AsyncStorage.getItem('eventQueue');
  if (storedEventQueue) {
    instance.eventQueue = JSON.parse(storedEventQueue);
    await flush(); // Try to flush stored events on app launch
  }

  startFlushTimer();

  if (instance.trackScreenViews) {
    startScreenTracking();
  }

  trackLifeCycleEvents();
  startNetworkListener();
};

let flushTimer: NodeJS.Timeout | null = null;

// new
const startFlushTimer = () => {
  if (flushTimer) {
    clearInterval(flushTimer);
  }

  flushTimer = setInterval(() => {
    flush();
  }, instance?.flushInterval || DEFAULT_FLUSH_INTERVAL);
};

export const getIdentityId = async () => {
  try {
    const identityId = await AsyncStorage.getItem('identityId');
    return identityId;
  } catch (error) {
    console.log(`Error while fetching identityId`, error?.message);
    return null;
  }
};

export const getOptOutStatus = async (): Promise<boolean> => {
  try {
    const storedStatus = await AsyncStorage.getItem('optOutStatus');
    return storedStatus ? JSON.parse(storedStatus) : false;
  } catch (error) {
    console.error('Error fetching opt-out status', error);
    return false;
  }
};

export const generateAnonymousId = (): string => {
  const timestamp = Date.now().toString(36);
  const randomString = Math.random().toString(36).slice(2, 6);
  const randomLetters =
    String.fromCharCode(65 + Math.floor(Math.random() * 26)) +
    String.fromCharCode(97 + Math.floor(Math.random() * 26));
  return `${randomLetters}-${timestamp}-${randomString}`;
};
